/*
  Simple DirectMedia Layer
  Copyright (C) 1997-2015 Sam Lantinga <slouken@libsdl.org>

  This software is provided 'as-is', without any express or implied
  warranty.  In no event will the authors be held liable for any damages
  arising from the use of this software.

  Permission is granted to anyone to use this software for any purpose,
  including commercial applications, and to alter it and redistribute it
  freely, subject to the following restrictions:

  1. The origin of this software must not be misrepresented; you must not
     claim that you wrote the original software. If you use this software
     in a product, an acknowledgment in the product documentation would be
     appreciated but is not required.
  2. Altered source versions must be plainly marked as such, and must not be
     misrepresented as being the original software.
  3. This notice may not be removed or altered from any source distribution.
*/
/* common_float.h */

/* This file provides a set of C functions for dealing with floating-point */
/*  numbers. */


#ifndef COMMON_FLOAT_H

# define COMMON_FLOAT_H



# ifndef NAMESPACE
#  error "common_float.h requires the macro NAMESPACE to be defined."
# else


	/* For FLT_EPSILON, DBL_EPSILON and LDBL_EPSILON. */
# include <float.h>



# include "common_macros.h"



#  if defined( USE_FLT ) || ( !defined( USE_DBL ) && !defined( USE_LDBL ) )

#   define FLT_TYPE float
#   define EPSILON FLT_EPSILON

#  elif defined( USE_DBL )

#   define FLT_TYPE double
#   define EPSILON DBL_EPSILON

#  else

#   define FLT_TYPE long double
#   define EPSILON LDBL_EPSILON

#  endif



#  define CEIL CONCAT( NAMESPACE, ceil )
#  define FLOOR CONCAT( NAMESPACE, floor )

#  define EQ CONCAT( NAMESPACE, r_eq )
#  define GR CONCAT( NAMESPACE, r_gr )
#  define LS CONCAT( NAMESPACE, r_ls )
#  define GREQ CONCAT( NAMESPACE, r_greq )
#  define LSEQ CONCAT( NAMESPACE, r_lseq )

#  define TWEEN CONCAT( NAMESPACE, tween )

#  define INT_CONSTR CONCAT( NAMESPACE, int_constraints )



/* These are designed to provide PROPER floating-point comparisons. */

int CEIL( FLT_TYPE in,    long *result );
int FLOOR( FLT_TYPE in,    long *result );

	/* Simple comparisons. */
int EQ ( FLT_TYPE a, FLT_TYPE b,  FLT_TYPE range );
int GR ( FLT_TYPE a, FLT_TYPE b,  FLT_TYPE range );
int LS ( FLT_TYPE a, FLT_TYPE b,  FLT_TYPE range );

	/* Compound comparisons. */
int GREQ ( FLT_TYPE a, FLT_TYPE b,  FLT_TYPE range );
int LSEQ ( FLT_TYPE a, FLT_TYPE b,  FLT_TYPE range );


	/* Arbitrary-element interpretation. */
int TWEEN ( FLT_TYPE *f1,  FLT_TYPE *f2,    FLT_TYPE *fdest,    size_t elem_count,  FLT_TYPE twval );


	/* Arbitrary-element bounds finding. */
int INT_CONSTR ( size_t vertcount, FLT_TYPE *vals,    long *ldest, long *hdest );

# endif

#endif
/* End common_float.h */
